"""add classifier and watchlist tables

Revision ID: 11311da39745
Revises: e81292491222
Create Date: 2018-09-21 15:44:16.007631

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "11311da39745"
down_revision = "e81292491222"
branch_labels = None
depends_on = None


def upgrade():
    # clear out everything
    op.execute("TRUNCATE TABLE feeds CASCADE")

    op.create_table(
        "watchlists",
        sa.Column("id", sa.String(), nullable=False),
        sa.Column("org_id", sa.String(), nullable=False),
        sa.Column("name", sa.String(), nullable=True),
        sa.Column("tags_enabled", sa.Boolean(), nullable=False),
        sa.Column("alerts_enabled", sa.Boolean(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )

    op.drop_constraint("reports_feed_id_fkey", "reports", type_="foreignkey")
    # op.drop_column('reports', 'feed_id', cascade=True)
    op.execute("ALTER TABLE reports DROP COLUMN feed_id CASCADE")
    op.create_primary_key("reports_pkey", "reports", ["id"])

    op.add_column("iocs", sa.Column("field", sa.String(), nullable=True))
    op.add_column("iocs", sa.Column("id", sa.String(), nullable=False))
    op.add_column("iocs", sa.Column("link", sa.String(), nullable=True))
    op.add_column("iocs", sa.Column("match_type", sa.String(), nullable=True))
    op.add_column("iocs", sa.Column("values", sa.String(), nullable=True))
    op.create_foreign_key(
        None,
        "iocs",
        "reports",
        ["report_id"],
        ["id"],
        ondelete="CASCADE",
        deferrable=True,
    )
    op.drop_column("iocs", "entries")
    op.drop_column("iocs", "feed_id")
    op.create_primary_key("iocs_pkey", "iocs", ["id", "report_id"])

    op.add_column(
        "ignore_report_iocs", sa.Column("watchlist_id", sa.String(), nullable=False)
    )
    op.create_foreign_key(
        "ignore_report_iocs_ioc_id_fkey1",
        "ignore_report_iocs",
        "iocs",
        ["ioc_id", "report_id"],
        ["id", "report_id"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        None,
        "ignore_report_iocs",
        "watchlists",
        ["watchlist_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("ignore_report_iocs", "feed_id")
    op.drop_column("ignore_report_iocs", "org_id")
    op.create_primary_key(
        "ignore_report_iocs_pkey",
        "ignore_report_iocs",
        ["watchlist_id", "report_id", "ioc_id"],
    )

    op.add_column(
        "ignore_reports", sa.Column("watchlist_id", sa.String(), nullable=False)
    )
    op.create_foreign_key(
        None, "ignore_reports", "reports", ["report_id"], ["id"], ondelete="CASCADE"
    )
    op.create_foreign_key(
        None,
        "ignore_reports",
        "watchlists",
        ["watchlist_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("ignore_reports", "feed_id")
    op.drop_column("ignore_reports", "org_id")
    op.create_primary_key(
        "ignore_reports_pkey", "ignore_reports", ["watchlist_id", "report_id"]
    )

    op.create_foreign_key(
        None,
        "report_severity_overrides",
        "reports",
        ["report_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.drop_column("report_severity_overrides", "feed_id")
    op.create_primary_key(
        "report_severity_overrides_pkey",
        "report_severity_overrides",
        ["org_id", "report_id"],
    )

    op.create_table(
        "classifiers",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(), nullable=True),
        sa.Column("value", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("key", "value", name="uc_key_value_classifier"),
    )

    op.create_table(
        "classifier_watchlists",
        sa.Column("watchlist_id", sa.String(), nullable=False),
        sa.Column("classifier_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["classifier_id"], ["classifiers.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["watchlist_id"], ["watchlists.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("watchlist_id"),
    )

    op.create_table(
        "report_classifiers",
        sa.Column("report_id", sa.String(), nullable=False),
        sa.Column("classifier_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["classifier_id"], ["classifiers.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["report_id"], ["reports.id"], ondelete="CASCADE", deferrable=True
        ),
        sa.PrimaryKeyConstraint("report_id", "classifier_id"),
    )

    op.create_table(
        "report_watchlists",
        sa.Column("watchlist_id", sa.String(), nullable=False),
        sa.Column("report_id", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["report_id"], ["reports.id"], ondelete="CASCADE", deferrable=True
        ),
        sa.ForeignKeyConstraint(
            ["watchlist_id"], ["watchlists.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("watchlist_id", "report_id"),
    )

    op.create_table(
        "watchlist_augmentation_hashes",
        sa.Column("watchlist_id", sa.String(), nullable=False),
        sa.Column("hash", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["watchlist_id"], ["watchlists.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("watchlist_id"),
    )

    op.drop_table("subscriptions")
    op.drop_table("feed_augmentation_hashes")
    op.drop_table("iocs_v2")
    op.drop_table("alerts")


def downgrade():  # pragma: no cover
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "subscriptions",
        sa.Column("feed_owner", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.add_column(
        "subscriptions",
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.add_column(
        "subscriptions",
        sa.Column("feed_access", sa.VARCHAR(), autoincrement=False, nullable=True),
    )
    op.create_foreign_key(
        "subscriptions_feed_id_fkey",
        "subscriptions",
        "feeds",
        ["feed_id", "feed_access", "feed_owner"],
        ["id", "access", "owner"],
        onupdate="CASCADE",
        ondelete="CASCADE",
    )
    op.drop_column("subscriptions", "tags_enabled")
    op.drop_column("subscriptions", "id")
    op.drop_column("subscriptions", "alerts_enabled")
    op.add_column(
        "reports",
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.create_foreign_key(
        "reports_feed_id_fkey",
        "reports",
        "feeds",
        ["feed_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.add_column(
        "report_severity_overrides",
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.drop_constraint(None, "report_severity_overrides", type_="foreignkey")
    op.create_foreign_key(
        "report_severity_overrides_report_id_fkey",
        "report_severity_overrides",
        "reports",
        ["report_id", "feed_id"],
        ["id", "feed_id"],
        ondelete="CASCADE",
    )
    op.add_column(
        "iocs", sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False)
    )
    op.add_column(
        "iocs", sa.Column("entries", sa.VARCHAR(), autoincrement=False, nullable=True)
    )
    op.drop_constraint(None, "iocs", type_="foreignkey")
    op.create_foreign_key(
        "iocs_report_id_fkey",
        "iocs",
        "reports",
        ["report_id", "feed_id"],
        ["id", "feed_id"],
        ondelete="CASCADE",
        deferrable=True,
    )
    op.drop_column("iocs", "values")
    op.drop_column("iocs", "match_type")
    op.drop_column("iocs", "link")
    op.drop_column("iocs", "id")
    op.drop_column("iocs", "field")
    op.add_column(
        "ignore_reports",
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.drop_constraint(None, "ignore_reports", type_="foreignkey")
    op.create_foreign_key(
        "ignore_reports_report_id_fkey",
        "ignore_reports",
        "reports",
        ["report_id", "feed_id"],
        ["id", "feed_id"],
        ondelete="CASCADE",
    )
    op.drop_column("ignore_reports", "subscription_id")
    op.add_column(
        "ignore_report_iocs",
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
    )
    op.drop_constraint(None, "ignore_report_iocs", type_="foreignkey")
    op.create_foreign_key(
        "ignore_report_iocs_ioc_id_fkey",
        "ignore_report_iocs",
        "iocs_v2",
        ["ioc_id", "report_id", "feed_id"],
        ["id", "report_id", "feed_id"],
        ondelete="CASCADE",
    )
    op.create_table(
        "alerts",
        sa.Column("org_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["feed_id"], ["feeds.id"], name="alerts_feed_id_fkey", ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("org_id", "feed_id", name="alerts_pkey"),
    )
    op.create_table(
        "iocs_v2",
        sa.Column("id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("report_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("match_type", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column("field", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column("values", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.Column("link", sa.VARCHAR(), autoincrement=False, nullable=True),
        sa.ForeignKeyConstraint(
            ["report_id", "feed_id"],
            ["reports.id", "reports.feed_id"],
            name="iocs_v2_report_id_fkey",
            ondelete="CASCADE",
            deferrable=True,
        ),
        sa.PrimaryKeyConstraint("id", "report_id", "feed_id", name="iocs_v2_pkey"),
    )
    op.create_table(
        "feed_augmentation_hashes",
        sa.Column("org_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("feed_id", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.Column("hash", sa.VARCHAR(), autoincrement=False, nullable=False),
        sa.ForeignKeyConstraint(
            ["feed_id"],
            ["feeds.id"],
            name="feed_augmentation_hashes_feed_id_fkey",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint(
            "org_id", "feed_id", name="feed_augmentation_hashes_pkey"
        ),
    )
    op.drop_table("subscription_augmentation_hashes")
    op.drop_table("report_subscriptions")
    op.drop_table("report_classifiers")
    op.drop_table("classifier_subscriptions")
    op.drop_table("classifiers")
    # ### end Alembic commands ###
